@@ -1,8 +1,9 @@
 typedef struct {
 #ifdef USE_ITHREADS
-    pTHX;
+    PerlInterpreter *my_perl;
 #endif
     CV *callback; // can be NULL
+    SV *err;
 } sort_ctx_t;
 
 static int
@@ -10,9 +11,11 @@ shvxs_sort_cmp(const void *a, const void *b, void *ctx)
 {
     sort_ctx_t *c = (sort_ctx_t *)ctx;
 #ifdef USE_ITHREADS
-    dTHXa(c->aTHX);
+    dTHXa(c->my_perl);
 #endif
 
+    if (c->err) return 0;
+
     SV *sv_a = *(SV * const *)a;
     SV *sv_b = *(SV * const *)b;
 
@@ -28,15 +31,29 @@ shvxs_sort_cmp(const void *a, const void *b, void *ctx)
     SAVETMPS;
 
     PUSHMARK(SP);
+    EXTEND(SP, 2);
     XPUSHs(sv_a);
     XPUSHs(sv_b);
     PUTBACK;
 
-    I32 count = call_sv((SV *)c->callback, G_SCALAR);
+    I32 count = call_sv((SV *)c->callback, G_SCALAR | G_EVAL);
     SPAGAIN;
 
-    if ( count > 0 ) {
-        result = (int)SvIV(POPs);
+    if (SvTRUE(ERRSV)) {
+        c->err = newSVsv(ERRSV);
+        sv_setsv(ERRSV, &PL_sv_undef);
+        PUTBACK;
+        FREETMPS;
+        LEAVE;
+        return 0;
+    }
+
+    SV *ret = (count > 0) ? POPs : &PL_sv_undef;
+    if (!SvOK(ret)) {
+        result = 0;
+    }
+    else {
+        result = (int)SvIV(ret);
     }
 
     PUTBACK;
@@ -51,7 +68,7 @@ shvxs_sort_cmp(const void *a, const void *b, void *ctx)
 static int __cdecl
 shvxs_sort_cmp_win32(void *ctx, const void *a, const void *b) {
     sort_ctx_t *c = (sort_ctx_t *)ctx;
-    return shvxs_sort_cmp(c, a, b);
+    return shvxs_sort_cmp(a, b, ctx);
 }
 
 #define SHVXS_QSORT(base, n, size, ctx) qsort_s((base), (n), (size), shvxs_sort_cmp_win32, (ctx))
@@ -61,7 +78,7 @@ shvxs_sort_cmp_win32(void *ctx, const void *a, const void *b) {
 static int
 shvxs_sort_cmp_bsd(void *ctx, const void *a, const void *b) {
     sort_ctx_t *c = (sort_ctx_t *)ctx;
-    return xs_sort_cmp(c, a, b);
+    return shvxs_sort_cmp(a, b, ctx);
 }
 
 #define SHVXS_QSORT(base, n, size, ctx) qsort_r((base), (n), (size), (ctx), shvxs_sort_cmp_bsd)
